@@ -15,31 +15,14 @@ from UM.JobQueue import JobQueue
 #   use of the JobQueue for the actual threading.
 #   \sa JobQueue
 @signalemitter
-class Job():
-    ##  Initialize.
-    #
-    #   \param kwargs Keyword arguments.
-    #                 Possible keywords:
-    #                 - description \type{string} A short description of the job that can be displayed in the UI. Defaults to an empty string.
-    #                 - visible \type{bool} True if this job should be shown in the UI, False if not. Defaults to False
-    def __init__(self, **kwargs):
+class Job:
+    def __init__(self):
         super().__init__()
-        self._running = False
-        self._finished = False
-        self._result = None
-        self._description = kwargs.get("description", "")
-        self._visible = kwargs.get("visible", False)
+        self._running = False   # type: bool
+        self._finished = False  # type: bool
+        self._result = None     # type: any
         self._error = None
 
-    ##  Get the description for this job.
-    def getDescription(self):
-        return self._description
-
-    ##  Should this job be shown in the UI
-    #   \return \type{bool}
-    def isVisible(self):
-        return self._visible
-
     ##  Perform the actual task of this job. Should be reimplemented by subclasses.
     #   \exception NotImplementedError
     def run(self):
@@ -54,7 +37,7 @@ class Job():
     ##  Set the result of this job.
     #
     #   This should be called by run() to set the actual result of the Job.
-    def setResult(self, result):
+    def setResult(self, result: any):
         self._result = result
 
     ##  Set an exception that was thrown while the job was being executed.
@@ -63,7 +46,7 @@ class Job():
     #   to execute properly.
     #
     #   \param error \type{Exception} The exception to set.
-    def setError(self, error):
+    def setError(self, error: Exception):
         self._error = error
 
     ##  Start the job.
@@ -84,25 +67,25 @@ class Job():
     ##  Check whether the job is currently running.
     #
     #   \return \type{bool}
-    def isRunning(self):
+    def isRunning(self) -> bool:
         return self._running
 
     ##  Check whether the job has finished processing.
     #
     #   \return \type{bool}
-    def isFinished(self):
+    def isFinished(self) -> bool:
         return self._finished
 
     ##  Check whether the Job has encountered an error during execution.
     #
     #   \return \type{bool} True if an error was set, False if not.
-    def hasError(self):
+    def hasError(self) -> bool:
         return self._error is not None
 
     ##  Get the error that was encountered during execution.
     #
     #   \return \type{Exception} The error encountered during execution or None if there was no error.
-    def getError(self):
+    def getError(self) -> Exception:
         return self._error
 
     ##  Emitted when the job has finished processing.
