@@ -5,8 +5,11 @@ from UM.Math.Vector import Vector
 from UM.Math.AxisAlignedBox import AxisAlignedBox
 from UM.Logger import Logger
 from UM.Math import NumPyUtil
+from UM.Math.Matrix import Matrix
 
 from enum import Enum
+from typing import List, Optional
+
 import threading
 import numpy
 import numpy.linalg
@@ -26,6 +29,7 @@ class MeshType(Enum):
 # value to set a field to in set().
 Reuse = object()
 
+
 ##  Class to hold a list of verts and possibly how (and if) they are connected.
 #
 #   This class stores three numpy arrays that contain the data for a mesh. Vertices
@@ -47,16 +51,16 @@ class MeshData:
         self._vertex_count = len(self._vertices) if self._vertices is not None else 0
         self._face_count = len(self._indices) if self._indices is not None else 0
         self._type = type
-        self._file_name = file_name
+        self._file_name = file_name  # type: str
         # original center position
         self._center_position = center_position
         # original zero position, is changed after transformation
         if zero_position is not None:
             self._zero_position = zero_position
         else:
-            self._zero_position = Vector(0, 0, 0)
-        self._convex_hull = None    # type: scipy.spatial.qhull.ConvexHull
-        self._convex_hull_vertices = None
+            self._zero_position = Vector(0, 0, 0) # type: Vector
+        self._convex_hull = None    # type: Optional[scipy.spatial.ConvexHull]
+        self._convex_hull_vertices = None  # type: Optional[numpy.ndarray]
         self._convex_hull_lock = threading.Lock()
 
         self._attributes = {}
@@ -73,7 +77,7 @@ class MeshData:
     ## Create a new MeshData with specified changes
     #   \return \type{MeshData}
     def set(self, vertices=Reuse, normals=Reuse, indices=Reuse, colors=Reuse, uvs=Reuse, file_name=Reuse,
-            center_position=Reuse, zero_position=Reuse, attributes=Reuse):
+            center_position=Reuse, zero_position=Reuse, attributes=Reuse) -> "MeshData":
         vertices = vertices if vertices is not Reuse else self._vertices
         normals = normals if normals is not Reuse else self._normals
         indices = indices if indices is not Reuse else self._indices
@@ -92,24 +96,24 @@ class MeshData:
         m.update(self.getVerticesAsByteArray())
         return m.hexdigest()
 
-    def getCenterPosition(self):
+    def getCenterPosition(self) -> Vector:
         return self._center_position
 
-    def getZeroPosition(self):
+    def getZeroPosition(self) -> Vector:
         return self._zero_position
 
     def getType(self):
         return self._type
 
-    def getFaceCount(self):
+    def getFaceCount(self) -> int:
         return self._face_count
 
     ##  Get the array of vertices
-    def getVertices(self):
+    def getVertices(self) -> numpy.ndarray:
         return self._vertices
 
     ##  Get the number of vertices
-    def getVertexCount(self):
+    def getVertexCount(self) -> int:
         return self._vertex_count
 
     ##  Get a vertex by index
@@ -120,37 +124,37 @@ class MeshData:
             return None
 
     ##  Return whether this mesh has vertex normals.
-    def hasNormals(self):
+    def hasNormals(self) -> bool:
         return self._normals is not None
 
     ##  Return the list of vertex normals.
-    def getNormals(self):
+    def getNormals(self) -> numpy.ndarray:
         return self._normals
 
     ##  Return whether this mesh has indices.
-    def hasIndices(self):
+    def hasIndices(self) -> bool:
         return self._indices is not None
 
     ##  Get the array of indices
     #   \return \type{numpy.ndarray}
-    def getIndices(self):
+    def getIndices(self) -> numpy.ndarray:
         return self._indices
 
-    def hasColors(self):
+    def hasColors(self) -> bool:
         return self._colors is not None
 
-    def getColors(self):
+    def getColors(self) -> numpy.ndarray:
         return self._colors
 
-    def hasUVCoordinates(self):
+    def hasUVCoordinates(self) -> bool:
         return self._uvs is not None
 
-    def getFileName(self):
+    def getFileName(self) -> str:
         return self._file_name
 
     ##  Transform the meshdata, center and zero position by given Matrix
     #   \param transformation 4x4 homogenous transformation matrix
-    def getTransformed(self, transformation):
+    def getTransformed(self, transformation: Matrix) -> "MeshData":
         if self._vertices is not None:
             transformed_vertices = transformVertices(self._vertices, transformation)
             transformed_normals = transformNormals(self._normals, transformation) if self._normals is not None else None
@@ -169,7 +173,7 @@ class MeshData:
     ##  Get the extents of this mesh.
     #
     #   \param matrix The transformation matrix from model to world coordinates.
-    def getExtents(self, matrix = None):
+    def getExtents(self, matrix: Optional[Matrix] = None) -> Optional[AxisAlignedBox]:
         if self._vertices is None:
             return None
 
@@ -189,7 +193,7 @@ class MeshData:
     ##  Get all vertices of this mesh as a bytearray
     #
     #   \return A bytearray object with 3 floats per vertex.
-    def getVerticesAsByteArray(self):
+    def getVerticesAsByteArray(self) -> Optional[bytes]:
         if self._vertices is None:
             return None
         # FIXME cache result
@@ -198,7 +202,7 @@ class MeshData:
     ##  Get all normals of this mesh as a bytearray
     #
     #   \return A bytearray object with 3 floats per normal.
-    def getNormalsAsByteArray(self):
+    def getNormalsAsByteArray(self) -> Optional[bytes]:
         if self._normals is None:
             return None
         # FIXME cache result
@@ -207,19 +211,19 @@ class MeshData:
     ##  Get all indices as a bytearray
     #
     #   \return A bytearray object with 3 ints per face.
-    def getIndicesAsByteArray(self):
+    def getIndicesAsByteArray(self) -> Optional[bytes]:
         if self._indices is None:
             return None
         # FIXME cache result
         return self._indices.tostring()
 
-    def getColorsAsByteArray(self):
+    def getColorsAsByteArray(self) -> Optional[bytes]:
         if self._colors is None:
             return None
         # FIXME cache result
         return self._colors.tostring()
 
-    def getUVCoordinatesAsByteArray(self):
+    def getUVCoordinatesAsByteArray(self) -> Optional[bytes]:
         if self._uvs is None:
             return None
         # FIXME cache result
@@ -236,8 +240,8 @@ class MeshData:
 
     ##  Gets the Convex Hull of this mesh
     #
-    #    \return \type{scipy.spatial.qhull.ConvexHull}
-    def getConvexHull(self):
+    #    \return \type{scipy.spatial.ConvexHull}
+    def getConvexHull(self) -> Optional[scipy.spatial.ConvexHull]:
         with self._convex_hull_lock:
             if self._convex_hull is None:
                 self._computeConvexHull()
@@ -246,7 +250,7 @@ class MeshData:
     ##  Gets the convex hull points
     #
     #   \return \type{numpy.ndarray} the vertices which describe the convex hull
-    def getConvexHullVertices(self):
+    def getConvexHullVertices(self) -> numpy.ndarray:
         if self._convex_hull_vertices is None:
             convex_hull = self.getConvexHull()
             self._convex_hull_vertices = numpy.take(convex_hull.points, convex_hull.vertices, axis=0)
@@ -255,44 +259,46 @@ class MeshData:
     ##  Gets transformed convex hull points
     #
     #   \return \type{numpy.ndarray} the vertices which describe the convex hull
-    def getConvexHullTransformedVertices(self, transformation):
+    def getConvexHullTransformedVertices(self, transformation: Matrix) -> numpy.ndarray:
         vertices = self.getConvexHullVertices()
         if vertices is not None:
             return transformVertices(vertices, transformation)
         else:
             return None
 
-    def hasAttribute(self, key):
+    def hasAttribute(self, key: str) -> bool:
         return key in self._attributes
 
     ##  the return value is a dict with at least keys opengl_name, opengl_type, value
-    def getAttribute(self, key):
+    def getAttribute(self, key: str):
         return self._attributes[key]
 
     ##  Return attribute names in alphabetical order
     #   The sorting assures that the order is always the same.
-    def attributeNames(self):
+    def attributeNames(self) -> List[str]:
         result = list(self._attributes.keys())
         result.sort()
         return result
 
-    def toString(self):
+    def toString(self) -> str:
         return "MeshData(_vertices=" + str(self._vertices) + ", _normals=" + str(self._normals) + ", _indices=" + \
                str(self._indices) + ", _colors=" + str(self._colors) + ", _uvs=" + str(self._uvs) + ", _attributes=" + \
                str(self._attributes.keys()) + ") "
 
+
 ##  Transform an array of vertices using a matrix
 #
 #   \param vertices \type{numpy.ndarray} array of 3D vertices
 #   \param transformation a 4x4 matrix
 #   \return \type{numpy.ndarray} the transformed vertices
-def transformVertices(vertices, transformation):
+def transformVertices(vertices: numpy.ndarray, transformation: Matrix) -> numpy.ndarray:
     data = numpy.pad(vertices, ((0, 0), (0, 1)), "constant", constant_values=(0.0, 0.0))
     data = data.dot(transformation.getTransposed().getData())
     data += transformation.getData()[:, 3]
     data = data[:, 0:3]
     return data
 
+
 ##  Transform an array of normals using a matrix
 #
 #   \param normals \type{numpy.ndarray} array of 3D normals
@@ -300,7 +306,7 @@ def transformVertices(vertices, transformation):
 #   \return \type{numpy.ndarray} the transformed normals
 #
 #   \note This assumes the normals are untranslated unit normals, and returns the same.
-def transformNormals(normals, transformation):
+def transformNormals(normals: numpy.ndarray, transformation: Matrix) -> numpy.ndarray:
     data = numpy.pad(normals, ((0, 0), (0, 1)), "constant", constant_values=(0.0, 0.0))
 
     # Get the translation from the transformation so we can cancel it later.
@@ -322,32 +328,35 @@ def transformNormals(normals, transformation):
 
     return data
 
+
 ##  Round an array of vertices off to the nearest multiple of unit
 #
 #   \param vertices \type{numpy.ndarray} the source array of vertices
 #   \param unit \type{float} the unit to scale the vertices to
 #   \return \type{numpy.ndarray} the rounded vertices
-def roundVertexArray(vertices, unit):
+def roundVertexArray(vertices: numpy.ndarray, unit: float) -> numpy.ndarray:
     expanded = vertices / unit
     rounded = expanded.round(0)
     return rounded * unit
 
+
 ##  Extract the unique vectors from an array of vectors
 #
 #   \param vertices \type{numpy.ndarray} the source array of vertices
 #   \return \type{numpy.ndarray} the array of unique vertices
-def uniqueVertices(vertices):
+def uniqueVertices(vertices: numpy.ndarray) -> numpy.ndarray:
     vertex_byte_view = numpy.ascontiguousarray(vertices).view(
         numpy.dtype((numpy.void, vertices.dtype.itemsize * vertices.shape[1])))
     _, idx = numpy.unique(vertex_byte_view, return_index=True)
     return vertices[idx]  # Select the unique rows by index.
 
+
 ##  Compute an approximation of the convex hull of an array of vertices
 #
 #   \param vertices \type{numpy.ndarray} the source array of vertices
 #   \param target_count \type{int} the maximum number of vertices which may be in the result
-#   \return \type{scipy.spatial.qhull.ConvexHull} the convex hull or None if the input was degenerate
-def approximateConvexHull(vertex_data, target_count):
+#   \return \type{scipy.spatial.ConvexHull} the convex hull or None if the input was degenerate
+def approximateConvexHull(vertex_data: numpy.ndarray, target_count: int) -> Optional[scipy.spatial.ConvexHull]:
     start_time = time()
 
     input_max = target_count * 50   # Maximum number of vertices we want to feed to the convex hull algorithm.
@@ -355,7 +364,14 @@ def approximateConvexHull(vertex_data, target_count):
 
     # Round off vertices and extract the uniques until the number of vertices is below the input_max.
     while len(vertex_data) > input_max:
-        vertex_data = uniqueVertices(roundVertexArray(vertex_data, unit_size))
+        new_vertex_data = uniqueVertices(roundVertexArray(vertex_data, unit_size))
+        # Check if there is variance in Z value, we need it for the convex hull calculation
+        if numpy.amin(new_vertex_data[:, 1]) != numpy.amax(new_vertex_data[:, 1]):
+            vertex_data = new_vertex_data
+        else:
+            # Prevent convex hull calculation from crashing
+            Logger.log("w", "Stopped shrinking data because otherwise the convex hull calculation will crash. Vertices: %s (target: %s)" % (len(vertex_data), input_max))
+            break
         unit_size *= 2
 
     if len(vertex_data) < 4:
@@ -376,12 +392,13 @@ def approximateConvexHull(vertex_data, target_count):
                target_count, end_time - start_time, len(vertex_data), len(hull_result.vertices))
     return hull_result
 
+
 ##  Calculate the normals of this mesh, assuming it was created by using addFace (eg; the verts are connected)
 #
 #   \param vertices \type{narray} list of vertices as a 1D list of float triples
 #   \param vertex_count \type{integer} the number of vertices to use in the vertices array
 #   \return \type{narray} list normals as a 1D array of floats, each group of 3 floats is a vector
-def calculateNormalsFromVertices(vertices, vertex_count):
+def calculateNormalsFromVertices(vertices: numpy.ndarray, vertex_count: int) -> numpy.ndarray:
     start_time = time()
     # Numpy magic!
 
@@ -408,22 +425,20 @@ def calculateNormalsFromVertices(vertices, vertex_count):
     Logger.log("d", "Calculating normals took %s seconds", end_time - start_time)
     return normals
 
+
 ## Calculate the normals of this mesh of triagles using indexes.
 #
 #   \param vertices \type{narray} list of vertices as a 1D list of float triples
 #   \param indices \type{narray} list of indices as a 1D list of integers
 #   \param face_count \type{integer} the number of triangles defined by the indices array
 #   \return \type{narray} list normals as a 1D array of floats, each group of 3 floats is a vector
-def calculateNormalsFromIndexedVertices(vertices, indices, face_count):
+def calculateNormalsFromIndexedVertices(vertices: numpy.ndarray, indices: numpy.ndarray, face_count: int) -> numpy.ndarray:
     start_time = time()
     # Numpy magic!
     # First, reset the normals
     normals = numpy.zeros((face_count*3, 3), dtype=numpy.float32)
 
     for face in indices[0:face_count]:
-        #print(self._vertices[face[0]])
-        #print(self._vertices[face[1]])
-        #print(self._vertices[face[2]])
         normals[face[0]] = numpy.cross(vertices[face[0]] - vertices[face[1]], vertices[face[0]] - vertices[face[2]])
         length = numpy.linalg.norm(normals[face[0]])
         normals[face[0]] /= length
