@@ -14,13 +14,11 @@ from . import BackendProxy
 from . import SceneProxy
 from . import ResourcesProxy
 from . import OperationStackProxy
-from . import JobsModel
 from UM.Mesh.MeshFileHandler import MeshFileHandler
 from UM.Workspace.WorkspaceFileHandler import WorkspaceFileHandler
 from . import PreferencesProxy
 from . import Theme
 from . import OpenGLContextProxy
-from . import AngledCornerRectangle
 from . import PointingRectangle
 from . import ActiveToolProxy
 from . import ActiveViewProxy
@@ -77,8 +75,6 @@ class Bindings:
         qmlRegisterType(MainWindow.MainWindow, "UM", 1, 0, "MainWindow")
         qmlRegisterType(ViewModel.ViewModel, "UM", 1, 0, "ViewModel")
         qmlRegisterType(ToolModel.ToolModel, "UM", 1, 0, "ToolModel")
-        qmlRegisterType(JobsModel.JobsModel, "UM", 1, 0, "JobsModel")
-        qmlRegisterType(AngledCornerRectangle.AngledCornerRectangle, "UM", 1, 0, "AngledCornerRectangle")
         qmlRegisterType(PointingRectangle.PointingRectangle, "UM", 1, 0, "PointingRectangle")
         qmlRegisterType(ExtensionModel.ExtensionModel, "UM", 1, 0, "ExtensionModel")
         qmlRegisterType(PluginsModel.PluginsModel, "UM", 1, 0, "PluginsModel")
