@@ -1,18 +1,20 @@
 # Copyright (c) 2015 Ultimaker B.V.
 # Uranium is released under the terms of the AGPLv3 or higher.
 
-from PyQt5.QtCore import pyqtProperty, QObject, Qt, QCoreApplication, pyqtSignal, pyqtSlot, QMetaObject, QRectF
+from PyQt5.QtCore import pyqtProperty, Qt, QCoreApplication, pyqtSignal, pyqtSlot, QMetaObject, QRectF
 from PyQt5.QtGui import QColor
-from PyQt5.QtQuick import QQuickWindow, QQuickItem
+from PyQt5.QtQuick import QQuickWindow
 
-from UM.Math.Vector import Vector
 from UM.Math.Matrix import Matrix
 from UM.Qt.QtMouseDevice import QtMouseDevice
 from UM.Qt.QtKeyDevice import QtKeyDevice
 from UM.Application import Application
 from UM.Preferences import Preferences
+from UM.Signal import Signal, signalemitter
+
 
 ##  QQuickWindow subclass that provides the main window.
+@signalemitter
 class MainWindow(QQuickWindow):
     def __init__(self, parent = None):
         super(MainWindow, self).__init__(parent)
@@ -20,7 +22,7 @@ class MainWindow(QQuickWindow):
         self._background_color = QColor(204, 204, 204, 255)
 
         self.setClearBeforeRendering(False)
-        self.beforeRendering.connect(self._render, type=Qt.DirectConnection)
+        self.beforeRendering.connect(self._render, type = Qt.DirectConnection)
 
         self._mouse_device = QtMouseDevice(self)
         self._mouse_device.setPluginId("qt_mouse")
@@ -44,6 +46,7 @@ class MainWindow(QQuickWindow):
         self.setWidth(int(self._preferences.getValue("general/window_width")))
         self.setHeight(int(self._preferences.getValue("general/window_height")))
         self.setPosition(int(self._preferences.getValue("general/window_left")), int(self._preferences.getValue("general/window_top")))
+
         # Make sure restored geometry is not outside the currently available screens
         screen_found = False
         for s in range(0, self._app.desktop().screenCount()):
@@ -51,7 +54,7 @@ class MainWindow(QQuickWindow):
                 screen_found = True
                 break
         if not screen_found:
-            self.setPosition(50,50)
+            self.setPosition(50, 50)
 
         self.setWindowState(int(self._preferences.getValue("general/window_state")))
         self._mouse_x = 0
@@ -65,9 +68,9 @@ class MainWindow(QQuickWindow):
     @pyqtSlot()
     def toggleFullscreen(self):
         if self._fullscreen:
-            self.setVisibility(QQuickWindow.Windowed) # Switch back to windowed
+            self.setVisibility(QQuickWindow.Windowed)  # Switch back to windowed
         else:
-            self.setVisibility(QQuickWindow.FullScreen) # Go to fullscreen
+            self.setVisibility(QQuickWindow.FullScreen)  # Go to fullscreen
         self._fullscreen = not self._fullscreen
 
     def getBackgroundColor(self):
@@ -110,7 +113,7 @@ class MainWindow(QQuickWindow):
         if event.isAccepted():
             return
 
-        if self.activeFocusItem() != None and self.activeFocusItem() != self._previous_focus:
+        if self.activeFocusItem() is not None and self.activeFocusItem() != self._previous_focus:
             self.activeFocusItem().setFocus(False)
 
         self._previous_focus = self.activeFocusItem()
@@ -155,7 +158,7 @@ class MainWindow(QQuickWindow):
         self._mouse_device.handleEvent(event)
 
     def moveEvent(self, event):
-        QMetaObject.invokeMethod(self, "_onWindowGeometryChanged", Qt.QueuedConnection);
+        QMetaObject.invokeMethod(self, "_onWindowGeometryChanged", Qt.QueuedConnection)
 
     def resizeEvent(self, event):
         super().resizeEvent(event)
@@ -165,11 +168,13 @@ class MainWindow(QQuickWindow):
 
         self._updateViewportGeometry(win_w, win_h)
 
-        QMetaObject.invokeMethod(self, "_onWindowGeometryChanged", Qt.QueuedConnection);
+        QMetaObject.invokeMethod(self, "_onWindowGeometryChanged", Qt.QueuedConnection)
 
     def hideEvent(self, event):
         Application.getInstance().windowClosed()
 
+    renderCompleted = Signal(type = Signal.Queued)
+
     def _render(self):
         renderer = self._app.getRenderer()
         view = self._app.getController().getActiveView()
@@ -179,6 +184,7 @@ class MainWindow(QQuickWindow):
         renderer.render()
         view.endRendering()
         renderer.endRendering()
+        self.renderCompleted.emit()
 
     def _onSceneChanged(self, object):
         self.update()
@@ -195,18 +201,18 @@ class MainWindow(QQuickWindow):
             self._preferences.setValue("general/window_state", Qt.WindowMaximized)
 
     def _updateViewportGeometry(self, width, height):
-        view_w = width * self._viewport_rect.width()
-        view_h = height * self._viewport_rect.height()
+        view_width = width * self._viewport_rect.width()
+        view_height = height * self._viewport_rect.height()
 
         for camera in self._app.getController().getScene().getAllCameras():
-            camera.setViewportSize(view_w, view_h)
+            camera.setViewportSize(view_width, view_height)
             camera.setWindowSize(width, height)
-            proj = Matrix()
+            projection_matrix = Matrix()
             if camera.isPerspective():
-                proj.setPerspective(30, view_w / view_h, 1, 500)
+                projection_matrix.setPerspective(30, view_width / view_height, 1, 500)
             else:
-                proj.setOrtho(-view_w / 2, view_w / 2, -view_h / 2, view_h / 2, -500, 500)
-            camera.setProjectionMatrix(proj)
+                projection_matrix.setOrtho(-view_width / 2, view_width / 2, -view_height / 2, view_height / 2, -500, 500)
+            camera.setProjectionMatrix(projection_matrix)
 
-        self._app.getRenderer().setViewportSize(view_w, view_h)
+        self._app.getRenderer().setViewportSize(view_width, view_height)
         self._app.getRenderer().setWindowSize(width, height)
