@@ -58,10 +58,11 @@ class OutputDeviceManagerProxy(QObject):
     def requestWriteToDevice(self, device_id, file_name, kwargs):
         limit_mimetypes = kwargs.get("limit_mimetypes", None)
         file_type = kwargs.get("file_type", "mesh")
+        preferred_mimetype = kwargs.get("preferred_mimetype", None)
         # On Windows, calling requestWrite() on LocalFileOutputDevice crashes when called from a signal
         # handler attached to a QML MenuItem. So instead, defer the call to the next run of the event 
         # loop, since that does work.
-        Application.getInstance().callLater(self._writeToDevice, [Application.getInstance().getController().getScene().getRoot()], device_id, file_name, limit_mimetypes, file_type)
+        Application.getInstance().callLater(self._writeToDevice, [Application.getInstance().getController().getScene().getRoot()], device_id, file_name, limit_mimetypes, file_type, preferred_mimetype = preferred_mimetype)
 
     ##  Request that the current selection is written to the output device.
     #
@@ -80,10 +81,11 @@ class OutputDeviceManagerProxy(QObject):
             return
 
         limit_mimetypes = kwargs.get("limit_mimetypes", False)
+        preferred_mimetype = kwargs.get("preferred_mimetype", None)
         # On Windows, calling requestWrite() on LocalFileOutputDevice crashes when called from a signal
         # handler attached to a QML MenuItem. So instead, defer the call to the next run of the event 
         # loop, since that does work.
-        Application.getInstance().callLater(self._writeToDevice, Selection.getAllSelectedObjects(), device_id, file_name, limit_mimetypes)
+        Application.getInstance().callLater(self._writeToDevice, Selection.getAllSelectedObjects(), device_id, file_name, limit_mimetypes, preferred_mimetype = preferred_mimetype)
 
     def _onActiveDeviceChanged(self):
         self.activeDeviceChanged.emit()
@@ -100,11 +102,10 @@ class OutputDeviceManagerProxy(QObject):
     #   to. Can be freely ignored if providing a file name makes no sense.
     #   \param limit_mimetypes: Limit the possible mimetypes to use for writing to these types.
     #   \param file_handler What file handler to get the writer from.
-    def _writeToDevice(self, nodes, device_id, file_name, limit_mimetypes, file_type = "mesh"):
+    def _writeToDevice(self, nodes, device_id, file_name, limit_mimetypes, file_type = "mesh", **kwargs):
         device = self._device_manager.getOutputDevice(device_id)
         if not device:
             return
-
         if file_type == "mesh":
             file_handler = Application.getInstance().getMeshFileHandler()
         elif file_type == "workspace":
@@ -114,7 +115,7 @@ class OutputDeviceManagerProxy(QObject):
             file_handler = None
 
         try:
-            device.requestWrite(nodes, file_name, limit_mimetypes, file_handler)
+            device.requestWrite(nodes, file_name, limit_mimetypes, file_handler, **kwargs)
         except OutputDeviceError.UserCanceledError:
             pass
         except OutputDeviceError.DeviceBusyError:
