@@ -12,11 +12,15 @@ class SelectionProxy(QObject):
         Selection.selectionChanged.connect(self._onSelectionChanged)
 
     selectionChanged = pyqtSignal()
-    
+
     @pyqtProperty(bool, notify = selectionChanged)
     def hasSelection(self):
         return Selection.hasSelection()
 
+    @pyqtProperty(int, notify = selectionChanged)
+    def selectionCount(self):
+        return Selection.getCount()
+
     def _onSelectionChanged(self):
         self.selectionChanged.emit()
 
