@@ -127,17 +127,17 @@ class Resources:
         if resource_type not in cls.__types:
             raise ResourceTypeError("Unknown type {0}".format(resource_type))
 
-        paths = []
+        paths = set()
 
         try:
-            paths.append(cls.getStoragePathForType(resource_type))
+            paths.add(cls.getStoragePathForType(resource_type))
         except UnsupportedStorageTypeError:
             pass
 
         for path in cls.__paths:
-            paths.append(os.path.join(path, cls.__types[resource_type]))
+            paths.add(os.path.join(path, cls.__types[resource_type]))
 
-        return paths
+        return list(paths)
 
     ##  Return a path where a certain resource type can be stored.
     #
@@ -498,4 +498,5 @@ class Resources:
         Cache: "",
         InstanceContainers: "instances",
         ContainerStacks: "stacks",
+        Themes: "themes",
     }
