@@ -57,7 +57,7 @@ class DefinitionContainer(QObject, DefinitionContainerInterface, PluginObject):
         super().__init__(parent = None, *args, **kwargs)
 
         self._id = str(container_id)    # type: str
-        self._name = container_id       # type: str
+        self._name = str(container_id)  # type: str
         self._metadata = {}             # type: Dict[str, Any]
         self._definitions = []          # type: List[SettingDefinition]
         self._inherited_files = []      # type: List[str]
@@ -167,7 +167,7 @@ class DefinitionContainer(QObject, DefinitionContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::getProperty
     #
     #   Reimplemented from ContainerInterface.
-    def getProperty(self, key, property_name):
+    def getProperty(self, key, property_name, context = None):
         definition = self._getDefinition(key)
         if not definition:
             return None
@@ -200,12 +200,18 @@ class DefinitionContainer(QObject, DefinitionContainerInterface, PluginObject):
     #   data about inheritance and overrides was lost when deserialising.
     #
     #   Reimplemented from ContainerInterface
-    def serialize(self):
+    def serialize(self, ignored_metadata_keys: Optional[List] = None):
         data = { } # The data to write to a JSON file.
         data["name"] = self.getName()
         data["version"] = DefinitionContainer.Version
         data["metadata"] = self.getMetaData()
 
+        # remove the keys that we want to ignore in the metadata
+        if ignored_metadata_keys:
+            for key in ignored_metadata_keys:
+                if key in data["metadata"]:
+                    del data["metadata"][key]
+
         data["settings"] = { }
         for definition in self.definitions:
             data["settings"][definition.key] = definition.serialize_to_dict()
@@ -388,8 +394,11 @@ class DefinitionContainer(QObject, DefinitionContainerInterface, PluginObject):
             return
 
         for setting in function.getUsedSettingKeys():
-            # Do not create relation on self
-            if setting == definition.key:
+            # Prevent circular relations between the same setting and the same property
+            # Note that the only property used by SettingFunction is the "value" property, which
+            # is why this is hard coded here.
+            if setting == definition.key and property == "value":
+                Logger.log("w", "Found circular relation for property 'value' between {0} and {1}", definition.key, setting)
                 continue
 
             other = self._getDefinition(setting)
