@@ -8,13 +8,6 @@ i18n_catalog = i18nCatalog("uranium")
 
 def getMetaData():
     return {
-        "plugin": {
-            "name": i18n_catalog.i18nc("@label", "Mirror Tool"),
-            "author": "Ultimaker",
-            "version": "1.0",
-            "description": i18n_catalog.i18nc("@info:whatsthis", "Provides the Mirror tool."),
-            "api": 3
-        },
         "tool": {
             "name": i18n_catalog.i18nc("@label", "Mirror"),
             "description": i18n_catalog.i18nc("@info:tooltip", "Mirror Model"),
