@@ -15,6 +15,7 @@ from UM.Operations.GroupedOperation import GroupedOperation
 from UM.Operations.SetTransformOperation import SetTransformOperation
 from UM.Operations.ScaleToBoundsOperation import ScaleToBoundsOperation
 
+from PyQt5.QtCore import Qt
 from . import ScaleToolHandle
 
 import scipy
@@ -37,6 +38,8 @@ class ScaleTool(Tool):
         self._maximum_bounds = None
         self._move_up = True
 
+        self._shortcut_key = Qt.Key_A
+
         # We use the position of the scale handle when the operation starts.
         # This is done in order to prevent runaway reactions (drag changes of 100+)
         self._saved_handle_position = None  # for non uniform drag
